@@ -16,7 +16,7 @@
 #
 #  partly based on work by Nokia Solutions and Networks Oyj
 """A parallel executor for Robot Framework test cases.
-Version 1.3.0
+Version 1.4.0
 
 Supports all Robot Framework command line options and also following
 options (these must be before normal RF options):
@@ -392,31 +392,6 @@ def _options_to_cli_arguments(opts): # type: (dict) -> List[str]
     return res
 
 
-class GatherSuiteNames(ResultVisitor):
-    def __init__(self):
-        self.result = [] # type: List[SuiteItem]
-
-    def end_suite(self, suite):
-        if len(suite.tests):
-            tests = [t.longname for t in suite.tests if 'pabot:dynamictest' not in t.tags]
-            dynamictests = [t.longname for t in suite.tests if 'pabot:dynamictest' in t.tags]
-            self.result.append(SuiteItem(suite.longname, tests=tests, dynamictests=dynamictests))
-
-
-def get_suite_names(output_file):
-    if not os.path.isfile(output_file):
-        print("get_suite_names: output_file='%s' does not exist" % output_file)
-        return []
-    try:
-        e = ExecutionResult(output_file)
-        gatherer = GatherSuiteNames()
-        e.visit(gatherer)
-        return gatherer.result
-    except:
-        print("Exception in get_suite_names!")
-        return []
-
-
 def _processes_count():
     try:
         return max(multiprocessing.cpu_count(), 2)
@@ -433,6 +408,8 @@ def _parse_args(args):
                   'pabotlibhost': '127.0.0.1',
                   'pabotlibport': 8270,
                   'processes': _processes_count(),
+                  'artifacts': ['png'],
+                  'artifactsinsubfolders': False,
                   'argumentfiles': []}
     while args and (args[0] in ['--' + param for param in ['hive',
                                                            'command',
@@ -445,6 +422,8 @@ def _parse_args(args):
                                                            'pabotlibport',
                                                            'ordering',
                                                            'suitesfrom',
+                                                           'artifacts',
+                                                           'artifactsinsubfolders',
                                                            'help']] or
                         ARGSMATCHER.match(args[0])):
         if args[0] == '--hive':
@@ -492,6 +471,14 @@ def _parse_args(args):
             pabot_args['suitesfrom'] = args[1]
             args = args[2:]
             continue
+        if args[0] == '--artifacts':
+            pabot_args['artifacts'] = args[1].split(',')
+            args = args[2:]
+            continue
+        if args[0] == '--artifactsinsubfolders':
+            pabot_args['artifactsinsubfolders'] = True
+            args = args[1:]
+            continue
         match = ARGSMATCHER.match(args[0])
         if match:
             pabot_args['argumentfiles'] += [(match.group(1), args[1])]
@@ -557,7 +544,7 @@ def _delete_none_keys(d):
 
 def hash_directory(digest, path):
     if os.path.isfile(path):
-        digest.update(_digest(path))
+        digest.update(_digest(_norm_path(path)))
         get_hash_of_file(path, digest)
         return
     for root, _, files in os.walk(path):
@@ -566,11 +553,14 @@ def hash_directory(digest, path):
             if os.path.isfile(file_path) and \
                 any(file_path.endswith(p) for p in _ROBOT_EXTENSIONS):
                 # DO NOT ALLOW CHANGE TO FILE LOCATION
-                digest.update(_digest(root))
+                digest.update(_digest(_norm_path(root)))
                 # DO THESE IN TWO PHASES BECAUSE SEPARATOR DIFFERS IN DIFFERENT OS
                 digest.update(_digest(name))
                 get_hash_of_file(file_path, digest)
 
+def _norm_path(path):
+    return "/".join(os.path.normpath(path).split(os.path.sep))
+
 def _digest(text):
     text = text.decode('utf-8') if PY2 and not is_unicode(text)  else text
     return hashlib.sha1(text.encode('utf-8')).digest()
@@ -1280,20 +1270,32 @@ def _output_dir(options, cleanup=True):
     return outpath
 
 
-def _copy_screenshots(options):
+def _copy_output_artifacts(options, file_extensions=None, include_subfolders=False):
+    file_extensions = file_extensions or ["png"]
     pabot_outputdir = _output_dir(options, cleanup=False)
     outputdir = options.get('outputdir', '.')
-    for location, dir_names, file_names in os.walk(pabot_outputdir):
+    copied_artifacts = []
+    for location, _, file_names in os.walk(pabot_outputdir):
         for file_name in file_names:
-            # We want ALL screenshots copied, not just selenium ones!
-            if file_name.endswith(".png"):
-                prefix = os.path.relpath(location, pabot_outputdir)
-                # But not .png files in any sub-folders of "location"
-                if os.sep in prefix:
-                    continue
+            file_ext = file_name.split(".")[-1]
+            if file_ext in file_extensions:
+                rel_path = os.path.relpath(location, pabot_outputdir)
+                prefix = rel_path.split(os.sep)[0]  # folders named "process-id"
+                dst_folder_path = outputdir
+                # if it is a file from sub-folders of "location"
+                if os.sep in rel_path:
+                    if not include_subfolders:
+                        continue
+                    # create destination sub-folder
+                    subfolder_path = rel_path[rel_path.index(os.sep)+1:]
+                    dst_folder_path = os.path.join(outputdir, subfolder_path)
+                    if not os.path.isdir(dst_folder_path):
+                        os.makedirs(dst_folder_path)
                 dst_file_name = '-'.join([prefix, file_name])
                 shutil.copyfile(os.path.join(location, file_name),
-                                os.path.join(outputdir, dst_file_name))
+                                os.path.join(dst_folder_path, dst_file_name))
+                copied_artifacts.append(file_name)
+    return copied_artifacts
 
 
 def _report_results(outs_dir, pabot_args, options, start_time_string, tests_root_name):
@@ -1304,16 +1306,17 @@ def _report_results(outs_dir, pabot_args, options, start_time_string, tests_root
     if pabot_args['argumentfiles']:
         outputs = [] # type: List[str]
         for index, _ in pabot_args['argumentfiles']:
+            copied_artifacts = _copy_output_artifacts(options, pabot_args['artifacts'], pabot_args['artifactsinsubfolders'])
             outputs += [_merge_one_run(os.path.join(outs_dir, index), options, tests_root_name, stats,
+                                    copied_artifacts,
                                     outputfile=os.path.join('pabot_results', 'output%s.xml' % index))]
-            _copy_screenshots(options)
         if 'output' not in options:
             options['output'] = 'output.xml'
         _write_stats(stats)
         return rebot(*outputs, **_options_for_rebot(options,
                                                     start_time_string, _now()))
     else:
-        return _report_results_for_one_run(outs_dir, options, start_time_string, tests_root_name, stats)
+        return _report_results_for_one_run(outs_dir, pabot_args, options, start_time_string, tests_root_name, stats)
 
 def _write_stats(stats):
     crit = stats['critical']
@@ -1322,9 +1325,9 @@ def _write_stats(stats):
     _write('%d tests total, %d passed, %d failed' % (al['total'], al['passed'], al['failed']))
     _write('===================================================')
 
-def _report_results_for_one_run(outs_dir, options, start_time_string, tests_root_name, stats):
-    output_path = _merge_one_run(outs_dir, options, tests_root_name, stats)
-    _copy_screenshots(options)
+def _report_results_for_one_run(outs_dir, pabot_args, options, start_time_string, tests_root_name, stats):
+    copied_artifacts = _copy_output_artifacts(options, pabot_args['artifacts'], pabot_args['artifactsinsubfolders'])
+    output_path = _merge_one_run(outs_dir, options, tests_root_name, stats, copied_artifacts)
     _write_stats(stats)
     if ('report' in options and options['report'] == "NONE" and
         'log' in options and options['log'] == "NONE"):
@@ -1335,7 +1338,7 @@ def _report_results_for_one_run(outs_dir, options, start_time_string, tests_root
     return rebot(output_path, **_options_for_rebot(options,
                                                    start_time_string, _now()))
 
-def _merge_one_run(outs_dir, options, tests_root_name, stats, outputfile='output.xml'):
+def _merge_one_run(outs_dir, options, tests_root_name, stats, copied_artifacts, outputfile='output.xml'):
     output_path = os.path.abspath(os.path.join(
         options.get('outputdir', '.'),
         options.get('output', outputfile)))
@@ -1348,7 +1351,7 @@ def _merge_one_run(outs_dir, options, tests_root_name, stats, outputfile='output
         _ABNORMAL_EXIT_HAPPENED = True
     if PY2:
         files = [f.decode(SYSTEM_ENCODING) if not is_unicode(f) else f for f in files]
-    resu = merge(files, options, tests_root_name, invalid_xml_callback)
+    resu = merge(files, options, tests_root_name, copied_artifacts, invalid_xml_callback)
     _update_stats(resu, stats)
     resu.save(output_path)
     return output_path
