@@ -149,6 +149,9 @@ class PabotLib(_PabotLib):
     __version__ = 0.67
     ROBOT_LIBRARY_SCOPE = 'GLOBAL'
     ROBOT_LISTENER_API_VERSION = 2
+    _pollingSeconds_SetupTeardown = 0.3
+    _pollingSeconds = 0.1
+    _polling_logging = True
 
     def __init__(self):
         _PabotLib.__init__(self)
@@ -218,6 +221,25 @@ class PabotLib(_PabotLib):
             self.__remotelib = Remote(uri) if uri else None
         return self.__remotelib
 
+    def set_polling_seconds(self, secs):
+        """
+        Determine the amount of seconds to wait between checking for free locks. Default: 0.1  (100ms)
+        """
+        PabotLib._pollingSeconds = secs
+
+    def set_polling_seconds_setupteardown(self, secs):
+        """
+        Determine the amount of seconds to wait between checking for free locks during setup and teardown. Default: 0.3  (300ms)
+        """
+        PabotLib._pollingSeconds_SetupTeardown = secs
+
+    def set_polling_logging(self, enable):
+        """
+        Enable or disable logging inside of polling. Logging inside of polling can be disabled (enable=False) to reduce log file size.
+        """
+        if isinstance(enable,str): enable = (enable.lower()=='true')
+        PabotLib._polling_logging = bool(enable)
+
     def run_setup_only_once(self, keyword, *args):
         """
         Runs a keyword only once at the first possible moment when
@@ -280,8 +302,8 @@ class PabotLib(_PabotLib):
         logger.trace("Queue index (%d)" % queue_index)
         if self._remotelib:
             while self.get_parallel_value_for_key(PABOT_MIN_QUEUE_INDEX_EXECUTING_PARALLEL_VALUE) < queue_index:
-                logger.trace(self.get_parallel_value_for_key(PABOT_MIN_QUEUE_INDEX_EXECUTING_PARALLEL_VALUE))
-                time.sleep(0.3)
+                if PabotLib._polling_logging: logger.trace(self.get_parallel_value_for_key(PABOT_MIN_QUEUE_INDEX_EXECUTING_PARALLEL_VALUE))
+                time.sleep(PabotLib._pollingSeconds_SetupTeardown)
         logger.trace("Teardown conditions met. Executing keyword.")
         BuiltIn().run_keyword(keyword, *args)
 
@@ -297,7 +319,7 @@ class PabotLib(_PabotLib):
         queue_index = int(BuiltIn().get_variable_value('${%s}' % PABOT_QUEUE_INDEX) or 0)
         if queue_index > 0 and self._remotelib:
             while self.get_parallel_value_for_key('pabot_only_last_executing') != 1:
-                time.sleep(0.3)
+                time.sleep(PabotLib._pollingSeconds_SetupTeardownp)
         BuiltIn().run_keyword(keyword)
 
     def set_parallel_value_for_key(self, key, value):
@@ -335,8 +357,8 @@ class PabotLib(_PabotLib):
             try:
                 while not self._remotelib.run_keyword('acquire_lock',
                                                       [name, self._my_id], {}):
-                    time.sleep(0.1)
-                    logger.debug('waiting for lock to release')
+                    time.sleep(PabotLib._pollingSeconds)
+                    if PabotLib._polling_logging: logger.debug('waiting for lock to release')
                 return True
             except RuntimeError:
                 logger.error('No connection - is pabot called with --pabotlib option?')
@@ -377,8 +399,8 @@ class PabotLib(_PabotLib):
                     if self._setname:
                         logger.info('Value set "%s" acquired' % self._setname)
                         return self._setname
-                    time.sleep(0.1)
-                    logger.debug('waiting for a value set')
+                    time.sleep(PabotLib._pollingSeconds)
+                    if PabotLib._polling_logging: logger.debug('waiting for a value set')
             except RuntimeError:
                 logger.error('No connection - is pabot called with --pabotlib option?')
                 self.__remotelib = None
