@@ -35,13 +35,20 @@ from robot.model import SuiteVisitor
 
 class ResultMerger(SuiteVisitor):
 
-    def __init__(self, result, tests_root_name):
+    def __init__(self, result, tests_root_name, out_dir, copied_artifacts):
         self.root = result.suite
         self.errors = result.errors
         self.current = None
         self._skip_until = None
         self._tests_root_name = tests_root_name
         self._prefix = ""
+        self._out_dir = out_dir
+
+        self._patterns = []
+        regexp_template = r'(src|href)="(.*?[\\\/]+)?({})"'  # https://regex101.com/r/sBwbgN/5
+        for artifact in copied_artifacts:
+            pattern = regexp_template.format(re.escape(artifact))
+            self._patterns.append(re.compile(pattern))
 
     def merge(self, merged):
         try:
@@ -128,15 +135,24 @@ class ResultMerger(SuiteVisitor):
         cur.starttime = min([cur.starttime, suite.starttime])
 
     def visit_message(self, msg):
-        if msg.html and re.search(r'src="([^"]+\.png)"', msg.message):
-            parent = msg.parent
-            while not isinstance(parent, TestSuite):
-                parent = parent.parent
-            suites_names = parent.longname.split('.')
-            if self._tests_root_name:
-                suites_names[0] = self._tests_root_name
-            msg.message = re.sub(r'"([^"]+\.png)"',
-                                 r'"%s-\1"' % self._prefix, msg.message)
+        if not msg.html:  # no html -> no link -> no update needed
+            return
+        if not self._patterns:  # don't update links if no artifacts were copied
+            return
+        if not ("src=" in msg.message or "href=" in msg.message):  # quick check before start search with complex regex
+            return
+
+        for pattern in self._patterns:
+            match = re.search(pattern, msg.message)
+            if match:
+                parent = msg.parent
+                while not isinstance(parent, TestSuite):
+                    parent = parent.parent
+                suites_names = parent.longname.split('.')
+                if self._tests_root_name:
+                    suites_names[0] = self._tests_root_name
+                msg.message = msg.message[:match.start(3)] + self._prefix + "-" + msg.message[match.start(3):]
+                # group 3 of regexp is the file name
 
 
 class ResultsCombiner(CombinedResult):
@@ -168,26 +184,27 @@ def group_by_root(results, critical_tags, non_critical_tags, invalid_xml_callbac
     return groups
 
 
-def merge_groups(results, critical_tags, non_critical_tags, tests_root_name, invalid_xml_callback):
+def merge_groups(results, critical_tags, non_critical_tags, tests_root_name, invalid_xml_callback, out_dir, copied_artifacts):
     merged = []
     for group in group_by_root(results, critical_tags,
                                non_critical_tags, invalid_xml_callback).values():
         base = group[0]
-        merger = ResultMerger(base, tests_root_name)
+        merger = ResultMerger(base, tests_root_name, out_dir, copied_artifacts)
         for out in group:
             merger.merge(out)
         merged.append(base)
     return merged
 
 
-def merge(result_files, rebot_options, tests_root_name, invalid_xml_callback=None):
+def merge(result_files, rebot_options, tests_root_name, copied_artifacts, invalid_xml_callback=None):
     assert(len(result_files) > 0)
     if invalid_xml_callback is None:
         invalid_xml_callback = lambda:0
     settings = RebotSettings(rebot_options)
     merged = merge_groups(result_files, settings.critical_tags,
                           settings.non_critical_tags, tests_root_name,
-                          invalid_xml_callback)
+                          invalid_xml_callback, settings.output_directory,
+                          copied_artifacts)
     if len(merged) == 1:
         if not merged[0].suite.doc:
             merged[0].suite.doc = '[https://pabot.org/?ref=log|Pabot] result from %d executions.' % len(result_files)
